"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stream_1 = require("stream");
const json_mapper_1 = require("./json-mapper");
const _ = require("lodash");
function createInstance(clazz, data = {}) {
    return new clazz(data);
}
class MappingStream extends stream_1.Transform {
    constructor(options) {
        super(options || {});
        this.mapping = options.mapping;
        this.clazz = options.clazz;
        this.jsonMapper = new json_mapper_1.JsonMapper();
        this.index = 0;
        if (!this.mapping) {
            throw new Error('No mapping supplied for mapping stream');
        }
        if (!this.clazz) {
            throw new Error('No clazz supplied for mapping streams');
        }
    }
    _transform(rawJson, encoding, next) {
        const mappedJson = this.jsonMapper.mapJson(rawJson, this.mapping);
        const mappedObject = createInstance(this.clazz, mappedJson);
        const prunedObject = this._pruneBlankValues(mappedObject);
        // TODO: Is valid?
        this.push(prunedObject);
        this.index++;
        next();
    }
    // TODO: Change this to prune object in-place using delete
    _pruneBlankValues(obj = {}) {
        Object.keys(obj).forEach((key, index) => {
            if (_.isObject(obj[key]) && !_.isDate(obj[key]) && !_.isArray(obj[key])) {
                obj[key] = this._pruneBlankValues(obj[key]);
            }
            else if (_.isNil(obj[key])) {
                delete obj[key];
            }
        });
        return obj;
    }
}
exports.MappingStream = MappingStream;
